import classNames from 'classnames'
import { useMemo } from 'react'
import { useLocation } from 'react-router-dom'

import Skeleton from 'components/account/AccountDetails/Skeleton'
import AccountSummary from 'components/account/AccountSummary'
import AccountSummaryLeverage from 'components/account/AccountSummary/AccountSummaryLeverage'
import { HealthGauge } from 'components/account/Health/HealthGauge'
import useBorrowMarketAssetsTableData from 'components/borrow/Table/useBorrowMarketAssetsTableData'
import { glowElement } from 'components/common/Button/utils'
import DisplayCurrency from 'components/common/DisplayCurrency'
import { FormattedNumber } from 'components/common/FormattedNumber'
import { ThreeDots } from 'components/common/Icons'
import Text from 'components/common/Text'
import useLendingMarketAssetsTableData from 'components/earn/lend/Table/useLendingMarketAssetsTableData'
import { DEFAULT_SETTINGS } from 'constants/defaultSettings'
import { LocalStorageKeys } from 'constants/localStorageKeys'
import { ORACLE_DENOM } from 'constants/oracle'
import useAccountIds from 'hooks/accounts/useAccountIds'
import useAccounts from 'hooks/accounts/useAccounts'
import useCurrentAccount from 'hooks/accounts/useCurrentAccount'
import useAllAssets from 'hooks/assets/useAllAssets'
import useLocalStorage from 'hooks/localStorage/useLocalStorage'
import useAccountId from 'hooks/useAccountId'
import useHealthComputer from 'hooks/useHealthComputer'
import useHLSStakingAssets from 'hooks/useHLSStakingAssets'
import usePrices from 'hooks/usePrices'
import useVaultAprs from 'hooks/vaults/useVaultAprs'
import useStore from 'store'
import { BNCoin } from 'types/classes/BNCoin'
import {
  calculateAccountApr,
  calculateAccountBalanceValue,
  calculateAccountLeverage,
} from 'utils/accounts'

export default function AccountDetailsController() {
  const address = useStore((s) => s.address)
  const isHLS = useStore((s) => s.isHLS)
  const { data: _, isLoading } = useAccounts('default', address)
  const { data: accountIds } = useAccountIds(address, false, true)

  const accountId = useAccountId()

  const account = useCurrentAccount()
  const focusComponent = useStore((s) => s.focusComponent)
  const isOwnAccount = accountId && accountIds?.includes(accountId)

  if (!address || focusComponent || !isOwnAccount || isHLS) return null

  if ((isLoading && accountId && !focusComponent) || !account) return <Skeleton />

  return <AccountDetails account={account} />
}

interface Props {
  account: Account
}

function AccountDetails(props: Props) {
  const { account } = props
  const location = useLocation()
  const { data: hlsStrategies } = useHLSStakingAssets()
  const { data: vaultAprs } = useVaultAprs()
  const [reduceMotion] = useLocalStorage<boolean>(
    LocalStorageKeys.REDUCE_MOTION,
    DEFAULT_SETTINGS.reduceMotion,
  )
  const updatedAccount = useStore((s) => s.updatedAccount)
  const accountDetailsExpanded = useStore((s) => s.accountDetailsExpanded)
  const { health, healthFactor } = useHealthComputer(account)
  const { health: updatedHealth, healthFactor: updatedHealthFactor } = useHealthComputer(
    updatedAccount || account,
  )
  const { data: prices } = usePrices()
  const assets = useAllAssets()
  const accountBalanceValue = useMemo(
    () => calculateAccountBalanceValue(updatedAccount ?? account, prices, assets),
    [updatedAccount, account, prices, assets],
  )
  const coin = BNCoin.fromDenomAndBigNumber(ORACLE_DENOM, accountBalanceValue)
  const leverage = useMemo(
    () => calculateAccountLeverage(account, prices, assets),
    [account, assets, prices],
  )
  const updatedLeverage = useMemo(() => {
    if (!updatedAccount) return null
    const updatedLeverage = calculateAccountLeverage(updatedAccount, prices, assets)

    if (updatedLeverage.eq(leverage)) return null
    return updatedLeverage
  }, [updatedAccount, prices, leverage, assets])

  const data = useBorrowMarketAssetsTableData()
  const borrowAssetsData = useMemo(() => data?.allAssets || [], [data])

  const { availableAssets: lendingAvailableAssets, accountLentAssets } =
    useLendingMarketAssetsTableData()

  const lendingAssetsData = useMemo(
    () => [...lendingAvailableAssets, ...accountLentAssets],
    [lendingAvailableAssets, accountLentAssets],
  )
  const apr = useMemo(
    () =>
      calculateAccountApr(
        updatedAccount ?? account,
        borrowAssetsData,
        lendingAssetsData,
        prices,
        hlsStrategies,
        assets,
        vaultAprs,
        account.kind === 'high_levered_strategy',
      ),
    [
      account,
      assets,
      borrowAssetsData,
      hlsStrategies,
      lendingAssetsData,
      prices,
      updatedAccount,
      vaultAprs,
    ],
  )
  const isFullWidth =
    location.pathname.includes('trade') ||
    location.pathname === '/' ||
    location.pathname.includes('perps')

  return (
    <>
      {!isFullWidth && accountDetailsExpanded && (
        <div
          className='absolute w-full h-full hover:cursor-pointer z-1'
          onClick={() => useStore.setState({ accountDetailsExpanded: false })}
        />
      )}
      <div
        data-testid='account-details'
        className={classNames(
          accountDetailsExpanded ? 'right-4' : '-right-74',
          'w-94 flex items-start gap-4 absolute top-6 z-2',
          !reduceMotion && 'transition-all duration-500',
        )}
      >
        <div
          className={classNames(
            'group/accountdetail relative min-h-75',
            'border rounded-base border-white/20',
            'backdrop-blur-sticky z-3 overflow-hidden',
            !reduceMotion && 'transition-all duration-500',
            accountDetailsExpanded
              ? 'is-expanded w-full h-auto'
              : 'w-16 hover:bg-white/10 hover:cursor-pointer bg-white/5',
          )}
          onClick={() => {
            if (accountDetailsExpanded) return
            useStore.setState({ accountDetailsExpanded: true })
          }}
        >
          <div
            className={classNames(
              'w-16 pr-[1px]',
              accountDetailsExpanded
                ? 'opacity-0 absolute inset-0 -z-1'
                : 'transition-opacity opacity-100 duration-300 delay-200',
            )}
          >
            <div className='flex flex-wrap justify-center w-full py-4'>
              <HealthGauge
                health={health}
                updatedHealth={updatedHealth}
                healthFactor={healthFactor}
                updatedHealthFactor={updatedHealthFactor}
              />
              <Text size='2xs' className='mb-0.5 mt-1 w-full text-center text-white/50'>
                Health
              </Text>
            </div>
            <div className='w-full py-4 border-t border-white/20'>
              <Text
                size='2xs'
                className='mb-0.5 w-full text-center text-white/50 whitespace-nowrap'
              >
                Net worth
              </Text>
              <DisplayCurrency coin={coin} className='w-full text-center truncate text-2xs ' />
            </div>
            <div className='w-full py-4 border-t border-white/20'>
              <Text size='2xs' className='mb-0.5 w-full text-center text-white/50'>
                Leverage
              </Text>
              <AccountSummaryLeverage
                leverage={leverage.toNumber() || 1}
                updatedLeverage={updatedLeverage?.toNumber() || null}
              />
            </div>
            <div className='w-full py-4 border-t border-white/20'>
              <Text size='2xs' className='mb-0.5 w-full text-center text-white/50'>
                APR
              </Text>
              <FormattedNumber
                className={'w-full text-center text-2xs'}
                amount={apr.toNumber()}
                options={{ maxDecimals: 2, minDecimals: 2, suffix: '%' }}
                animate
              />
            </div>
          </div>
          <div
            className={classNames(
              'grid relative z-2',
              !reduceMotion && 'transition-[grid-template-rows,opacity]',
              accountDetailsExpanded
                ? 'transition-[grid-template-rows,opacity] opacity-100 delay-200 duration-600 grid-rows-[1fr]'
                : 'transition-opacity opacity-0 duration-300 grid-rows-[0fr]',
            )}
          >
            <div className='overflow-hidden'>
              <AccountSummary account={account} isAccountDetails />
            </div>
            <div
              className={classNames(
                'absolute inset-0 -z-1',
                'before:content-[""] before:transition-opacity before:-z-1 before:absolute before:left-0 before:h-full before:w-full before:bg-white/10 before:rounded-b-base before:border-t before:border-white/10',
                accountDetailsExpanded
                  ? 'before:opacity-100 before:delay-500 before:top-full'
                  : 'before:opacity-0 before:duration-0',
              )}
            />
          </div>
          <div
            className={classNames(
              'flex justify-center items-center w-full h-6 opacity-50',
              !reduceMotion && 'transition-[opacity] duration-300',
              'absolute -bottom-6',
              'group-hover/accountdetail:opacity-100',
            )}
          >
            {!accountDetailsExpanded && <ThreeDots className='h-1' />}
          </div>

          {glowElement(!reduceMotion)}
        </div>
      </div>
    </>
  )
}
